from datetime import datetime, timedelta
from typing import Dict, List


class TimeProgramDaySetting:
    startTime: str = None
    temperature: float = None
    mode: str = None
    absoluteMinutes: int = None

    def __init__(self, start_time: str, temperature: float, mode: str):
        self.startTime = start_time
        self.temperature = temperature
        self.mode = mode
        self.absoluteMinutes = TimeProgramDaySetting.to_absolute_minute(start_time)

    @classmethod
    def to_absolute_minute(cls, start_time):
        split = start_time.split(":")
        if len(split) > 1:
            hour = int(split[0]) * 60
            minute = int(split[1])
            return hour + minute
        return 0


class TimeProgramDay:
    timeProgramDaySettings: List[TimeProgramDaySetting] = []

    def __init__(self):
        self.timeProgramDaySettings = list()

    def add_setting(self, start_time: str, temperature: float, mode: str):
        self.timeProgramDaySettings.append(TimeProgramDaySetting(start_time, temperature, mode))


class TimeProgram:
    timeProgramDays: Dict[str, TimeProgramDay] = dict()

    def __init__(self):
        self.timeProgramDays = dict()

    def add_day(self, day: str, time_program_day: TimeProgramDay):
        self.timeProgramDays[day] = time_program_day

    def get_current_time_program(self, search_date: datetime):
        day = search_date.strftime("%A").lower()
        day_before = (search_date - timedelta(days=1)).strftime("%A").lower()
        time = str(search_date.hour) + ":" + str(search_date.minute)

        absolute_minute = TimeProgramDaySetting.to_absolute_minute(time)
        timeProgramDay = self.timeProgramDays[day]
        timeProgramDayBefore = self.timeProgramDays[day_before]

        """if hour:minute is before the first setting of the day, check the last of day -1"""
        if absolute_minute < timeProgramDay.timeProgramDaySettings[0].absoluteMinutes:
            return timeProgramDayBefore.timeProgramDaySettings[-1]
        else:
            idx = 0
            while idx < len(timeProgramDay.timeProgramDaySettings) - 1:
                if absolute_minute > timeProgramDay.timeProgramDaySettings[idx].absoluteMinutes and \
                        (idx + 1 == len(timeProgramDay.timeProgramDaySettings)
                         or absolute_minute < timeProgramDay.timeProgramDaySettings[idx + 1].absoluteMinutes):
                    return timeProgramDay.timeProgramDaySettings[idx]
                idx += 1

            if idx == 0:
                return timeProgramDay.timeProgramDaySettings[0]
            elif idx == len(timeProgramDay.timeProgramDaySettings) - 1:
                return timeProgramDay.timeProgramDaySettings[-1]
        return None



class BoostMode:
    name: str = None
    forZone: bool = False
    forRoom: bool = False
    forWaterHeater: bool = False
    forCirculation: bool = False

    def __init__(self, name, for_zone, for_room, for_water_heater, for_circulation):
        self.name = name
        self.forZone = for_zone
        self.forRoom = for_room
        self.forWaterHeater = for_water_heater
        self.forCirculation = for_circulation

    @staticmethod
    def from_name(name: str):
        return _VALUES[name]


QM_HOTWATER_BOOST = BoostMode('QM_HOTWATER_BOOST', False, False, True, False)
QM_VENTILATION_BOOST = BoostMode('QM_VENTILATION_BOOST', True, True, False, False)
QM_ONE_DAY_AWAY = BoostMode('QM_ONE_DAY_AWAY', True, True, True, False)
QM_SYSTEM_OFF = BoostMode('QM_SYSTEM_OFF', True, True, True, True)
QM_ONE_DAY_AT_HOME = BoostMode('QM_ONE_DAY_AT_HOME', True, False, False, False)
QM_PARTY = BoostMode('QM_PARTY', True, False, False, False)

_VALUES = {
        QM_HOTWATER_BOOST.name: QM_HOTWATER_BOOST,
        QM_VENTILATION_BOOST.name: QM_VENTILATION_BOOST,
        QM_ONE_DAY_AWAY.name: QM_ONE_DAY_AWAY,
        QM_SYSTEM_OFF.name: QM_SYSTEM_OFF,
        QM_ONE_DAY_AT_HOME.name: QM_ONE_DAY_AT_HOME,
        QM_PARTY.name: QM_PARTY
    }






from . import BoostMode


class QuickMode:
    boostMode: BoostMode = None
    remainingDuration: int = 0

    def __init__(self, name: str, remaining_duration: int):
        self.boostMode = BoostMode.from_name(name)
        self.remainingDuration = remaining_duration



from datetime import datetime

from .constant import THERMOSTAT_QUICK_VETO
from . import TimeProgram, TimeProgramDaySetting, QuickVeto


class Component:
    id: str = None
    name: str = None
    timeProgram: TimeProgram = None
    currentTemperature: float = None
    targetTemperature: float = None
    operationMode: str = None
    quickVeto: QuickVeto = None

    def get_current_time_program(self):
        if self.quickVeto:
            return TimeProgramDaySetting('0',
                                         self.quickVeto.targetTemperature, THERMOSTAT_QUICK_VETO)

        return self.timeProgram.get_current_time_program(datetime.now())



import os
import pickle
import logging

logger = logging.getLogger('FileUtils')


class FileUtils:
    @staticmethod
    def load_from_file(path):
        try:
            with open(path, "rb") as f:
                return pickle.load(f)
        except FileNotFoundError:
            logger.debug("File %s not found", path)
            return None
        except Exception as e:
            logger.error("Cannot open file: %s, error: %s", path, str(e))
            return None

    @staticmethod
    def save_to_file(data, filename, path):
        try:
            os.makedirs(path, exist_ok=True)
            with open(path + "/" + filename, "wb+") as f:
                pickle.dump(data, f)
        except Exception as e:
            logger.error("Cannot save data: %s file: %s, error: ", path + filename, data, str(e))

    @staticmethod
    def delete_file(path):
        os.remove(path)

    @staticmethod
    def delete_dir(path):
        os.rmdir(path)



import copy
from typing import List

from .constant import THERMOSTAT_ZONE_MODE_OFF, THERMOSTAT_ROOM_MODE_MANUAL, FROST_PROTECTION_TEMP, \
    THERMOSTAT_ZONE_MODE_DAY
from . import Component, Room, TimeProgramDaySetting


class Zone(Component):
    targetMinTemperature: float = None
    activeFunction: str = None
    rooms: List[Room] = []
    rbr: bool = False

    def get_current_time_program(self):
        mode = copy.deepcopy(super().get_current_time_program())
        if self.quickVeto is None:
            if self.operationMode == THERMOSTAT_ZONE_MODE_OFF:
                mode = TimeProgramDaySetting(str(0), FROST_PROTECTION_TEMP, THERMOSTAT_ZONE_MODE_OFF)
            elif mode.mode == THERMOSTAT_ROOM_MODE_MANUAL:
                mode.temperature = self.targetTemperature
            elif mode.mode == THERMOSTAT_ZONE_MODE_DAY:
                mode.temperature = self.targetTemperature
            else:
                mode.temperature = self.targetMinTemperature
        return mode



import datetime

from . import *

DATE_FORMAT = "%Y-%m-%d"


class Mapper:

    @staticmethod
    def quick_mode(full_system):
        quick_mode = full_system.get("body").get("configuration", dict()).get("quickmode")
        if quick_mode:
            return QuickMode(quick_mode.get("quickmode"), quick_mode.get("duration"))

    @staticmethod
    def outdoor_temp(full_system):
        return full_system.get("body").get("status", dict()).get('outside_temperature')

    @staticmethod
    def installation_name(facilities):
        return facilities.get("body", dict()).get("facilitiesList", list())[0].get("name")

    @staticmethod
    def rooms(raw_rooms):
        rooms = list()
        if raw_rooms:
            for raw_room in raw_rooms.get("body", dict()).get("rooms"):
                rooms.append(Mapper.room(raw_room))

        return rooms

    @staticmethod
    def room(raw_room):
        room = Room()
        raw_room = raw_room.get("body") if raw_room.get("body") is not None else raw_room
        config = raw_room.get("configuration", dict())

        room.id = raw_room.get("roomIndex")
        room.childLock = config.get("childLock")
        room.targetTemperature = config.get("temperatureSetpoint")
        room.currentTemperature = config.get("currentTemperature")
        room.devices = Mapper.devices(config.get("devices"))
        room.isWindowOpen = config.get("isWindowOpen")
        room.name = config.get("name")
        room.operationMode = config.get("operationMode")

        raw_quickVeto = config.get("quickVeto")
        if raw_quickVeto:
            room.quickVeto = QuickVeto(raw_quickVeto.get("remainingDuration"), config.get("temperatureSetpoint"))

        room.timeProgram = Mapper.time_program(raw_room.get("timeprogram"))
        return room

    @staticmethod
    def devices(raw_devices):
        devices = list()
        if raw_devices:
            for raw_device in raw_devices:
                device = Device()
                device.name = raw_device["name"]
                device.deviceType = raw_device["deviceType"]
                device.isBatteryLow = raw_device["isBatteryLow"]
                device.isRadioOutOfReach = raw_device["isRadioOutOfReach"]
                device.sgtin = raw_device["sgtin"]
                devices.append(device)

        return devices

    @staticmethod
    def time_program(raw_time_program, mode_key_name=""):
        timeProgram = TimeProgram()
        if raw_time_program:
            timeProgram.add_day("monday", Mapper.time_program_day(raw_time_program.get("monday"), mode_key_name))
            timeProgram.add_day("tuesday", Mapper.time_program_day(raw_time_program.get("tuesday"), mode_key_name))
            timeProgram.add_day("wednesday", Mapper.time_program_day(raw_time_program.get("wednesday"), mode_key_name))
            timeProgram.add_day("thursday", Mapper.time_program_day(raw_time_program.get("thursday"), mode_key_name))
            timeProgram.add_day("friday", Mapper.time_program_day(raw_time_program.get("friday"), mode_key_name))
            timeProgram.add_day("saturday", Mapper.time_program_day(raw_time_program.get("saturday"), mode_key_name))
            timeProgram.add_day("sunday", Mapper.time_program_day(raw_time_program.get("sunday"), mode_key_name))

        return timeProgram

    @staticmethod
    def time_program_day(raw_time_program_day, mode_key_name=""):
        timeProgramDay: TimeProgramDay = TimeProgramDay()
        if raw_time_program_day:
            for time_setting in raw_time_program_day:
                timeProgramDay.add_setting(time_setting.get("startTime"), time_setting.get("temperatureSetpoint"),
                                           time_setting.get(mode_key_name))

        return timeProgramDay

    @staticmethod
    def holiday_mode(full_system):
        holidayMode = HolidayMode()
        holidayMode.active = False

        raw_holiday_mode = full_system.get("body", dict()).get("configuration", dict()).get("holidaymode")
        if raw_holiday_mode and raw_holiday_mode.get("active"):
            holidayMode.targetTemperature = raw_holiday_mode.get("temperature_setpoint")
            holidayMode.active = True
            holidayMode.startDate = datetime.datetime.strptime(raw_holiday_mode.get("start_date"), DATE_FORMAT).date()
            holidayMode.endDate = datetime.datetime.strptime(raw_holiday_mode.get("end_date"), DATE_FORMAT).date()

        return holidayMode

    @staticmethod
    def boiler_status(hvac_state, live_report):
        boilerStatus = BoilerStatus()

        hvac_state_info = Mapper.__find_hvac_message_status(hvac_state)
        if hvac_state_info:
            timestamp = hvac_state_info.get("timestamp")
            if timestamp:
                boilerStatus.lastUpdate = datetime.datetime.fromtimestamp(timestamp / 1000)

            boilerStatus.deviceName = hvac_state_info.get("deviceName")
            boilerStatus.code = hvac_state_info.get("statusCode")
            boilerStatus.title = hvac_state_info.get("title")
            boilerStatus.description = hvac_state_info.get("description")
            boilerStatus.hint = hvac_state_info.get("hint")

        water_pressure_report = Mapper.__find_water_pressure_report(live_report)
        if water_pressure_report:
            boilerStatus.waterPressure = water_pressure_report.get("value")
            boilerStatus.waterPressureUnit = water_pressure_report.get("unit")

        boiler_temp_report = Mapper.__find_boiler_temperature_report(live_report)
        if boiler_temp_report:
            boilerStatus.currentTemperature = boiler_temp_report.get("value")

        return boilerStatus

    @staticmethod
    def box_detail(facilities, system_status):
        pass
        # boxDetails = None
        #
        # facilityList = facilities.get("body", dict()).get("facilitiesList", list())
        #
        # if facilityList:
        #     boxDetails = BoxDetails()
        #
        #     details = facilityList[0]
        #     boxDetails.serialNumber = details.get("serialNumber")
        #     boxDetails.firmwareVersion = details.get("firmwareVersion")
        #
        #     network = details.get("networkInformation", dict())
        #     boxDetails.ethernetMac = network.get("macAddressEthernet")
        #     boxDetails.wifiMac = network.get("macAddressWifiClient")
        #     boxDetails.wifiAPMac = network.get("macAddressWifiAccessPoint")
        #
        #     boxDetails.onlineStatus = system_status.get("body", dict()).get("onlineStatus", dict()).get("status")
        #     boxDetails.updateStatus = system_status.get("body", dict()).
        #     get("firmwareUpdateStatus", dict()).get("status")
        #
        # return boxDetails

    @staticmethod
    def zones(full_system):
        zones = list()
        for raw_zone in full_system.get("body", dict()).get("zones", list()):
            zones.append(Mapper.zone(raw_zone))

        return zones

    @staticmethod
    def zone(raw_zone):
        zone = Zone()

        # meta = meta if meta is not None else raw_zone.get("meta", dict())
        zone.id = raw_zone.get("_id")

        heating = raw_zone.get("heating", dict())
        configuration = raw_zone.get("configuration", dict())
        heating_configuration = heating.get("configuration", dict())
        zone.operationMode = heating_configuration.get("mode")
        zone.targetTemperature = heating_configuration.get("setpoint_temperature")
        zone.targetMinTemperature = heating_configuration.get("setback_temperature")
        zone.timeProgram = Mapper.time_program(heating.get("timeprogram"), "setting")

        zone.name = configuration.get("name").strip()
        zone.currentTemperature = configuration.get("inside_temperature")
        zone.activeFunction = configuration.get("active_function")

        quickVeto = configuration.get("quick_veto")
        if quickVeto and quickVeto.get("active"):
            """No way to find start_date Quick veto on zone lasts 6 hours"""
            zone.quickVeto = QuickVeto(-1, quickVeto.get("setpoint_temperature"))

        zone.rbr = raw_zone.get("currently_controlled_by", dict()).get("name", "") == "RBR"

        return zone

    @staticmethod
    def domestic_hot_water(full_system, live_report):
        hotWater = None
        hot_water_list = full_system.get("body", dict()).get("dhw", list())

        if hot_water_list:
            raw_hot_water = hot_water_list[0].get("hotwater")
            if raw_hot_water:
                hotWater = HotWater()
                hotWater.targetTemperature = raw_hot_water.get("configuration", dict()).get("temperature_setpoint")
                hotWater.operationMode = raw_hot_water.get("configuration", dict()).get("operation_mode")
                hotWater.timeProgram = Mapper.time_program(raw_hot_water.get("timeprogram", dict()), "mode")
                hotWater.id = hot_water_list[0].get("_id")
                if hot_water_list[0].get("controlled_by"):
                    hotWater.operationMode = Mapper.quick_mode(full_system).boostMode.name

            dhw_report = Mapper.__find_dhw_temperature_report(live_report)

            if dhw_report:
                hotWater.currentTemperature = dhw_report.get("value")
                hotWater.name = dhw_report.get("name")

        return hotWater

    @staticmethod
    def circulation(full_system):
        circulation = None
        hot_water_list = full_system.get("body", dict()).get("dhw", list())

        if hot_water_list:
            raw_circulation = hot_water_list[0].get("circulation")
            if raw_circulation:
                circulation = Circulation()
                circulation.name = "Circulation"
                circulation.timeProgram = Mapper.time_program(raw_circulation.get("timeprogram", "setting"))
                circulation.operationMode = raw_circulation.get("configuration", dict()).get("operationMode")
                circulation.id = hot_water_list[0].get("_id")
                circulation.targetTemperature = 0
                circulation.currentTemperature = 0

        return circulation

    @staticmethod
    def __find_hvac_message_status(hvac_state):
        for message in hvac_state.get("body", dict()).get("errorMessages"):
            if message.get("type") == "STATUS":
                return message

        return None

    @staticmethod
    def __find_water_pressure_report(live_report):
        for device in live_report.get("body", dict()).get("devices", list()):
            for report in device.get("reports", list()):
                if report.get("associated_device_function") == "HEATING" and report.get("_id") == "WaterPressureSensor":
                    return report

        return None

    @staticmethod
    def __find_boiler_temperature_report(live_report):
        for device in live_report.get("body", dict()).get("devices", list()):
            for report in device.get("reports", list()):
                if report.get("associated_device_function") == "HEATING" \
                        and report.get("_id") == "FlowTemperatureSensor":
                    return report

        return None

    @staticmethod
    def __find_dhw_temperature_report(live_report):
        for device in live_report.get("body", dict()).get("devices", list()):
            for report in device.get("reports", list()):
                if report.get("associated_device_function") == "DHW" \
                        and report.get("_id") == "DomesticHotWaterTankTemperature":
                    return report

        return None

    @staticmethod
    def __find_zone_quick_veto_timestamp(zone_id, meta):
        for state in meta.get("resourceState", list()):
            if state.get("link", dict()).get("resourceLink", "") \
                    .find("/zones/" + zone_id + "/configuration/quick_veto"):
                return state.get("timestamp")
        return None

    @staticmethod
    def __find_dhw_quick_veto_timestamp(meta):
        for state in meta.get("resourceState", list()):
            if state.get("link", dict()).get("resourceLink", "") \
                    .find("/systemcontrol/v1/configuration/quickmode"):
                return state.get("timestamp")
        return None

    @staticmethod
    def __get_delta(start_time, time_to_add):
        end_time = start_time + time_to_add
        delta = (end_time / 1000) - datetime.datetime.now().timestamp()
        return int(delta / 60)



import tempfile

DEFAULT_FILES_DIR = tempfile.gettempdir() + "/vaillant_vr900_files"

DEFAULT_COOKIE_FILE_NAME = "/.vr900-vaillant.cookies"
DEFAULT_SERIAL_NUMBER_FILE_NAME = "/.vr900-vaillant.serial"

DEFAULT_SMARTPHONE_ID = "vr900-connector"

DEFAULT_BASE_URL = "https://smart.vaillant.com/mobile/api/v4"
TEST_LOGIN_URL = "/account/user/v1/details"
REQUEST_NEW_TOKEN_URL = "/account/authentication/v1/token/new"
AUTHENTICATE_URL = "/account/authentication/v1/authenticate"

FACILITIES_URL = "/facilities"
ROOMS_URL = "/facilities/{serialNumber}/rbr/v1/rooms"
ROOM_TIMEPROGRAM_URL = "/facilities/{serialNumber}/rbr/v1/rooms/{index}/timeprogram"
ZONES_URL = "/facilities/{serialNumber}/systemcontrol/v1/zones"
DHW_URL = "/facilities/{serialNumber}/systemcontrol/v1/dhw/{id}/hotwater"
DHW_SETPOINT_TEMPERATURE_URL = DHW_URL + '/configuration/temperature_setpoint'
DHW_SET_OPERATION_MODE_URL = DHW_URL + '/configuration/operation_mode'
CIRCULATION_URL = "/facilities/{serialNumber}/systemcontrol/v1/dhw/{id}/circulation"
HVAC_STATE_URL = "/facilities/{serialNumber}/hvacstate/v1/overview"
SYSTEM_STATUS_URL = "/facilities/{serialNumber}/system/v1/status"
SYSTEM_CONTROL_URL = "/facilities/{serialNumber}/systemcontrol/v1"
LIVE_REPORT_URL = "/facilities/{serialNumber}/livereport/v1"
QUICK_MODE_URL = SYSTEM_CONTROL_URL + "/configuration/quickmode"
CURRENT_PV_METERING_INFO_URL = '/facilities/{serialNumber}/spine/v1/currentPVMeteringInfo'
EMF_URL = '/facilities/{serialNumber}/emf/v1/devices'
REPEATERS_URL = '/facilities/{serialNumber}/rbr/v1/repeaters'



from datetime import datetime


class BoilerStatus:
    deviceName: str = None
    code: str = None
    title: str = None
    description: str = None
    hint: str = None
    lastUpdate: datetime = None
    currentTemperature: float = None
    waterPressure: float = None
    waterPressureUnit: str = None



HOLIDAY_MODE = 'HOLIDAY'
QM_HOTWATER_BOOST = 'QM_HOTWATER_BOOST'
QM_VENTILATION_BOOST = 'QM_VENTILATION_BOOST'
QM_ONE_DAY_AWAY = 'QM_ONE_DAY_AWAY'
QM_SYSTEM_OFF = 'QM_SYSTEM_OFF'
QM_ONE_DAY_AT_HOME = 'QM_ONE_DAY_AT_HOME'
QM_PARTY = 'QM_PARTY'

CIRCULATION_MODE_OFF = 'OFF'
CIRCULATION_MODE_ON = 'ON'
CIRCULATION_MODE_AUTO = 'AUTO'
CIRCULATION_MODE_AUTO_OFF = 'AUTO_OFF'
CIRCULATION_MODE_AUTO_ON = 'AUTO_ON'

HOT_WATER_MODE_AUTO = 'AUTO'
HOT_WATER_MODE_AUTO_OFF = 'AUTO_OFF'
HOT_WATER_MODE_AUTO_ON = 'AUTO_ON'
HOT_WATER_MODE_OFF = 'OFF'
HOT_WATER_MODE_ON = 'ON'
HOT_WATER_MODES = [HOT_WATER_MODE_AUTO, HOT_WATER_MODE_ON, HOT_WATER_MODE_OFF, QM_HOTWATER_BOOST]
HOT_WATER_MIN_TEMP = 35
HOT_WATER_MAX_TEMP = 70


THERMOSTAT_QUICK_VETO = "QUICK_VETO"
THERMOSTAT_ROOM_MODE_AUTO = 'AUTO'
THERMOSTAT_ROOM_MODE_AUTO_ON = 'AUTO_ON'
THERMOSTAT_ROOM_MODE_AUTO_OFF = 'AUTO_OFF'
THERMOSTAT_ROOM_MODE_OFF = 'OFF'
THERMOSTAT_ROOM_MODE_MANUAL = 'MANUAL'
THERMOSTAT_ROOM_MODES = [THERMOSTAT_ROOM_MODE_AUTO, THERMOSTAT_ROOM_MODE_OFF, THERMOSTAT_ROOM_MODE_MANUAL,
                         THERMOSTAT_QUICK_VETO]

THERMOSTAT_ZONE_MODE_OFF = 'OFF'
THERMOSTAT_ZONE_MODE_DAY = 'DAY'
THERMOSTAT_ZONE_MODE_NIGHT = 'NIGHT'
THERMOSTAT_ZONE_MODE_AUTO = 'AUTO'

THERMOSTAT_ZONE_MODES = [THERMOSTAT_ZONE_MODE_AUTO, THERMOSTAT_ZONE_MODE_OFF, THERMOSTAT_ZONE_MODE_DAY,
                         THERMOSTAT_ZONE_MODE_NIGHT, THERMOSTAT_QUICK_VETO]

THERMOSTAT_MAX_TEMP = 30
FROST_PROTECTION_TEMP = 5



import logging
from datetime import date

from .model.constant import QM_HOTWATER_BOOST
from .model import Room, Zone
from .model import Mapper
from .api import ApiConnector
from .api.constant import DEFAULT_SMARTPHONE_ID, DEFAULT_BASE_URL, DEFAULT_FILES_DIR
from .model import System, HotWater

LOGGER = logging.getLogger('VaillantSystemManager')


class VaillantSystemManager:

    def __init__(self, user, password, smartphone_id=DEFAULT_SMARTPHONE_ID,
                 base_url=DEFAULT_BASE_URL, file_dir=DEFAULT_FILES_DIR):
        self.__connector = ApiConnector(user, password, smartphone_id, base_url, file_dir)
        self.__mapper = Mapper()

    def get_system(self):
        try:
            self.__connector.autoCloseSession = False
            full_system = self.__connector.get_system_control()
            live_report = self.__connector.get_live_report()
            hvac_state = self.__connector.get_hvac_state()

            holiday_mode = self.__mapper.holiday_mode(full_system)
            boiler_status = self.__mapper.boiler_status(hvac_state, live_report)

            zones = self.__mapper.zones(full_system)

            for zone in zones:
                if zone.rbr:
                    raw_rooms = self.__connector.get_rooms()
                    rooms = self.__mapper.rooms(raw_rooms)
                    zone.rooms = rooms
                    break

            hot_water = self.__mapper.domestic_hot_water(full_system, live_report)
            circulation = self.__mapper.circulation(full_system)

            outdoorTemperature = self.__mapper.outdoor_temp(full_system)
            quickMode = self.__mapper.quick_mode(full_system)

            vaillant_system = System(holiday_mode, boiler_status, zones, hot_water, circulation,
                                     outdoorTemperature, quickMode)

            return vaillant_system
        finally:
            self.__connector.close_session()

    def refresh_room(self, room: Room):
        self.__connector.autoCloseSession = True
        return self.__mapper.room(self.__connector.get_room(room.id))

    def refresh_rooms(self):
        self.__connector.autoCloseSession = True
        return self.__mapper.rooms(self.__connector.get_rooms())

    def refresh_zones(self):
        self.__connector.autoCloseSession = True
        return self.__mapper.zones(self.__connector.get_zones())

    def refresh_zone(self, zone: Zone):
        self.__connector.autoCloseSession = True
        return self.__mapper.zones(self.__connector.get_zone(zone.id))

    def set_away(self, start_date: date, end_date: date):
        """TODO if not provided, default date, one day ?"""
        pass

    def remove_away(self):
        """TODO """
        pass

    def set_dhw_setpoint_temperature(self, hot_water: HotWater, temperature):
        LOGGER.info("Will try to set dhw target temperature to %s", temperature)
        if temperature:
            self.__connector.set_hot_water_operation_mode(hot_water.id, round(float(temperature)))
            return True
        else:
            LOGGER.debug("No temperature provided, nothing to do")
            return False

    """
        Returns True/False whether new_mode has been set or not.
        Throw an error if something went wrong while setting new_mode
    """

    def set_hot_water_operation_mode(self, sys: System, new_mode):
        LOGGER.info("Will try to set hot water mode to %s", new_mode)

        hot_water = sys.hotWater
        quick_mode = sys.quickMode
        if new_mode:
            if hot_water.operationMode != new_mode:
                if quick_mode:
                    if new_mode != QM_HOTWATER_BOOST:
                        LOGGER.debug("Quick mode %s is running and will get kept, new mode will be set",
                                     quick_mode.boostMode.name)
                        self.__connector.set_hot_water_operation_mode(hot_water.id, new_mode)
                        return True
                    else:
                        LOGGER.debug("Quick mode %s is running and new_mode is also quick mode, won't change",
                                     quick_mode.boostMode.name)
                        return False
                else:
                    if new_mode == QM_HOTWATER_BOOST:
                        LOGGER.debug("No quick mode running, "
                                     "new_mode is a quick mode and will be applied for the whole system")
                        self.__connector.set_quick_mode(new_mode)
                        return True
                    else:
                        LOGGER.debug("No quick mode running, new_mode is a classic mode")
                        self.__connector.set_hot_water_operation_mode(hot_water.id, new_mode)
                        return True
            else:
                LOGGER.debug("Mode %s is the same as previous mode %s", new_mode, hot_water.operationMode)
                return False
        else:
            LOGGER.debug("No new mode provided, nothing to do")
            return False

    def logout(self):
        self.__connector.close_session(True)



from requests import Response


class ApiError(Exception):
    message: str = None
    response: Response = None

    def __init__(self, message, response):
        self.message = message
        self.response = response



import copy

from .constant import CIRCULATION_MODE_ON, CIRCULATION_MODE_AUTO_ON, CIRCULATION_MODE_AUTO_OFF
from . import Component


class Circulation(Component):

    def get_current_time_program(self):
        """There is no quick veto for circulation"""
        mode = copy.deepcopy(super().get_current_time_program())
        if mode.mode == CIRCULATION_MODE_ON:
            mode.mode = CIRCULATION_MODE_AUTO_ON
        else:
            mode.mode = CIRCULATION_MODE_AUTO_OFF
        return mode



import copy

from .constant import HOT_WATER_MODE_OFF, QM_HOTWATER_BOOST, HOT_WATER_MODE_ON, \
    HOT_WATER_MODE_AUTO_OFF, HOT_WATER_MIN_TEMP, HOT_WATER_MODE_AUTO_ON
from . import Component, TimeProgramDaySetting


class HotWater(Component):

    def get_current_time_program(self):
        """There is no quick veto for hot water"""
        if self.operationMode == HOT_WATER_MODE_ON:
            return TimeProgramDaySetting(str(0), self.targetTemperature, HOT_WATER_MODE_ON)
        elif self.operationMode == HOT_WATER_MODE_OFF:
            return TimeProgramDaySetting(str(0), HOT_WATER_MIN_TEMP, HOT_WATER_MODE_OFF)
        elif self.operationMode == QM_HOTWATER_BOOST:
            return TimeProgramDaySetting(str(0), self.targetTemperature, QM_HOTWATER_BOOST)
        else:
            # Mode AUTO
            mode = copy.deepcopy(super().get_current_time_program())
            if mode.mode == HOT_WATER_MODE_ON:
                mode.mode = HOT_WATER_MODE_AUTO_ON
                mode.temperature = self.targetTemperature
            else:
                mode.mode = HOT_WATER_MODE_AUTO_OFF
                mode.temperature = HOT_WATER_MIN_TEMP
            return mode



import requests
import logging

from ..util.fileutils import FileUtils
from . import ApiError
from . import constant

LOGGER = logging.getLogger('Connector')

"""
This is the low level smart.vaillant.com API connector.
This connector is handling some part of the API as well as the login.
For now, only some GET part of the API are handled. It means you cannot alter data with this connector
"""


class ApiConnector:

    def __init__(self, user, password, smartphone_id=constant.DEFAULT_SMARTPHONE_ID, base_url=constant.DEFAULT_BASE_URL,
                 file_dir=constant.DEFAULT_FILES_DIR, auto_close_session=True):
        self.__user = user
        self.__password = password
        self.__smartphoneId = smartphone_id
        self.__baseUrl = base_url
        self.__fileDir = file_dir
        self.__headers = {"content-type": "application/json"}
        self.autoCloseSession = auto_close_session
        self.__serialNumber = self.__load_serial_number_from_file()
        self.__session = self.__create_session()

    """
    url must be relative to base_url.
    Serial number can be provided if you put {serialNumber} in url
    """
    def query(self, url, method='GET', payload=None):
        return self.__secure_call(method, url, payload)

    def get(self, url):
        return self.query(url)

    def put(self, url, payload=None):
        return self.query(url, 'PUT', payload)

    def post(self, url, payload=None):
        return self.query(url, 'POST', payload)

    def delete(self, url):
        return self.query(url, 'DELETE')

    def get_current_pv_metering_inf(self):
        return self.get(constant.CURRENT_PV_METERING_INFO_URL)

    def get_emf(self):
        return self.get(constant.EMF_URL)

    def get_repeaters(self):
        return self.get(constant.REPEATERS_URL)

    def get_facilities(self):
        return self.get(constant.FACILITIES_URL)

    def get_live_report(self):
        return self.get(constant.LIVE_REPORT_URL)

    def get_system_status(self):
        return self.get(constant.SYSTEM_STATUS_URL)

    def get_hvac_state(self):
        return self.get(constant.HVAC_STATE_URL)

    def get_rooms(self):
        return self.get(constant.ROOMS_URL)

    def get_system_control(self):
        return self.get(constant.SYSTEM_CONTROL_URL)

    def get_zones(self):
        return self.get(constant.ZONES_URL)

    def get_room(self, index):
        return self.get(constant.ROOMS_URL + "/" + str(index))

    def get_zone(self, zone_id):
        return self.get(constant.ZONES_URL + "/" + str(zone_id))

    def get_dhw(self, dhw_id):
        return self.get(self.__baseUrl + constant.DHW_URL.replace("{id}", str(dhw_id)))

    def get_circulation(self, dhw_id):
        return self.get(self.__baseUrl + constant.CIRCULATION_URL.replace("{id}", str(dhw_id)))

    def set_hot_water_setpoint_temperature(self, hot_water_id, temperature):
        return self.__secure_call('PUT', constant.DHW_SETPOINT_TEMPERATURE_URL.replace("{id}", str(hot_water_id)),
                                  {"temperature_setpoint": temperature})

    def set_hot_water_operation_mode(self, dhw_id, mode):
        return self.__secure_call('PUT', constant.DHW_SET_OPERATION_MODE_URL.replace("{id}", str(dhw_id)),
                                  {"operation_mode": mode})

    def remove_quick_mode(self):
        return self.__secure_call('DELETE', constant.QUICK_MODE_URL)

    def set_quick_mode(self, mode, duration=None):
        return self.__secure_call('PUT', constant.QUICK_MODE_URL,
                                  {"quickmode": {"quickmode": str(mode),
                                                 "duration": duration if duration is not None else 0}})

    def close_session(self, clear=False):
        LOGGER.debug("Closing session")
        self.__session.close()
        if clear:
            self.__clear_session()
            FileUtils.delete_dir(self.__fileDir)

    def __secure_call(self, method, url, payload=None):
        response = None
        try:
            self.__login()
            url = url.replace("{serialNumber}", self.__serialNumber)
            url = self.__baseUrl + url
            response = self.__session.request(method, url, json=payload,
                                              headers=None if payload is None else self.__headers)

            if response.status_code > 399:
                raise ApiError("Received error from server url: " + url + " and method " + method, response)
            if response.text:
                return response.json()
            else:
                return {"ok": "ok"}
        except ApiError:
            LOGGER.error("Cannot %s url: %s", method, url)
            raise
        except Exception as e:
            LOGGER.exception("Cannot %s url: %s", method, url)
            raise ApiError(str(e), response)
        finally:
            if self.autoCloseSession:
                self.close_session()

    def __login(self, relogin=False):
        try:
            if not self.__session.cookies:
                self.__session = self.__create_session()

                if not self.__session.cookies:
                    LOGGER.info(
                        "No previous session found, will try to logging with username: %s and smartphoneId: %s to %s",
                        self.__user, self.__smartphoneId, self.__baseUrl)

                    authtoken = self.__request_token()
                    self.__get_cookies(authtoken)
                    self.__get_serial_number()
            else:
                LOGGER.debug("Session already exists, will test it...")
                testLoginResponse = self.__test_login()
                if testLoginResponse.status_code != 200:
                    if relogin:
                        raise ApiError("Logging test failed", testLoginResponse)
                    else:
                        LOGGER.info("Cookie and serial files are outdated, re-logging")
                        self.__clear_session()
                        self.__login(True)
                else:
                    LOGGER.debug("... session is ok")
        except ApiError:
            raise
        except Exception as e:
            LOGGER.error("Error during logging", e)
            raise ApiError("Error during logging " + str(e), None)

    def __request_token(self):
        params = {
            "smartphoneId": self.__smartphoneId,
            "username": self.__user,
            "password": self.__password
        }

        response = self.__session.post(self.__baseUrl + constant.REQUEST_NEW_TOKEN_URL,
                                       json=params, headers=self.__headers)
        if response.status_code == 200:
            LOGGER.debug("Token generation successful")
            auth_token = response.json()["body"]["authToken"]
            if not auth_token:
                raise ApiError("Generated token is empty", response)
            return auth_token
        else:
            raise ApiError("Authentication failed", response)

    def __get_cookies(self, authtoken):
        params = {
            "smartphoneId": self.__smartphoneId,
            "username": self.__user,
            "authToken": authtoken
        }
        response = self.__session.post(self.__baseUrl + constant.AUTHENTICATE_URL, json=params, headers=self.__headers)

        if response.status_code == 200:
            LOGGER.debug("Cookie successfully retrieved")
            self.__save_cookies_to_file()
        else:
            raise ApiError("Cannot get cookies", response)

    def __get_serial_number(self):
        response = self.__session.get(self.__baseUrl + constant.FACILITIES_URL)

        if response.status_code == 200:
            LOGGER.debug("Serial number successfully retrieved")
            self.__serialNumber = response.json()["body"]["facilitiesList"][0]["serialNumber"]
            self.__save_serial_number_to_file()
        else:
            raise ApiError("Cannot get serial number", response)

    def __test_login(self):
        return self.__session.get(self.__baseUrl + constant.TEST_LOGIN_URL)

    def __create_session(self):
        session = requests.session()
        cookies = self.__load_cookies_from_file()
        LOGGER.debug("Found cookies %s", cookies)
        if cookies is not None:
            session.cookies = cookies
        return session

    def __clear_session(self):
        self.__clear_cookie()
        self.__clear_serial_numbr()
        self.__session = requests.session()

    def __save_cookies_to_file(self):
        FileUtils.save_to_file(self.__session.cookies, constant.DEFAULT_COOKIE_FILE_NAME, self.__fileDir)

    def __save_serial_number_to_file(self):
        FileUtils.save_to_file(self.__serialNumber, constant.DEFAULT_SERIAL_NUMBER_FILE_NAME, self.__fileDir)

    def __load_cookies_from_file(self):
        return FileUtils.load_from_file(self.__fileDir + constant.DEFAULT_COOKIE_FILE_NAME)

    def __load_serial_number_from_file(self):
        return FileUtils.load_from_file(self.__fileDir + constant.DEFAULT_SERIAL_NUMBER_FILE_NAME)

    def __clear_cookie(self):
        FileUtils.delete_file(self.__fileDir + '/' + constant.DEFAULT_COOKIE_FILE_NAME)

    def __clear_serial_numbr(self):
        FileUtils.delete_file(self.__fileDir + '/' + constant.DEFAULT_SERIAL_NUMBER_FILE_NAME)



import copy
from typing import List

from . import Component, Device, TimeProgramDaySetting
from .constant import FROST_PROTECTION_TEMP, THERMOSTAT_ROOM_MODE_OFF, \
    THERMOSTAT_ROOM_MODE_MANUAL, THERMOSTAT_ROOM_MODE_AUTO_OFF, THERMOSTAT_ROOM_MODE_AUTO_ON


class Room(Component):
    childLock: bool = False
    isWindowOpen: bool = False
    devices: List[Device] = []

    def get_current_time_program(self):
        mode = copy.deepcopy(super().get_current_time_program())
        if self.quickVeto is None:
            if self.operationMode == THERMOSTAT_ROOM_MODE_OFF:
                mode = TimeProgramDaySetting(str(0), FROST_PROTECTION_TEMP, THERMOSTAT_ROOM_MODE_OFF)
            elif mode.mode == THERMOSTAT_ROOM_MODE_MANUAL:
                mode.temperature = self.targetTemperature
            else:
                if mode.temperature >= self.targetTemperature:
                    mode.mode = THERMOSTAT_ROOM_MODE_AUTO_OFF
                else:
                    mode.mode = THERMOSTAT_ROOM_MODE_AUTO_ON
        return mode



class ActiveMode:
    targetTemperature: float = None
    name: str = None
    sub_mode: str = None

    def __init__(self, target_temperature: float, name: str, sub_mode: str = None):
        self.targetTemperature = target_temperature
        self.name = name
        self.sub_mode = sub_mode



class QuickVeto:
    remainingTime: int = 0
    targetTemperature: float = None

    def __init__(self, remaining_time, target_temperature):
        self.remainingTime = remaining_time
        self.targetTemperature = target_temperature



from typing import Dict

from . import constant, ActiveMode, BoilerStatus, Circulation, HolidayMode, HotWater, QuickMode, Room, Zone
from .boostmode import QM_SYSTEM_OFF, QM_HOTWATER_BOOST, QM_PARTY, QM_ONE_DAY_AT_HOME, QM_ONE_DAY_AWAY, \
    QM_VENTILATION_BOOST


class System:
    holidayMode: HolidayMode = HolidayMode()
    boilerStatus: BoilerStatus = None
    _zones: Dict[str, Zone] = dict()
    _rooms: Dict[int, Room] = dict()
    hotWater: HotWater = None
    circulation: Circulation = None
    outdoorTemperature: float = None
    quickMode: QuickMode = None

    def __init__(self, holiday_mode, boiler_status, zones, hot_water, circulation, outdoor_temp, quick_mode):
        if holiday_mode:
            self.holidayMode = holiday_mode
        self.boilerStatus = boiler_status

        if zones:
            self._zones = dict((zone.id, zone) for zone in zones)
            for zone in zones:
                if zone.rbr:
                    for room in zone.rooms:
                        room.zone = zone
                        self._rooms[room.id] = room

        self.hotWater = hot_water
        self.circulation = circulation
        self.outdoorTemperature = outdoor_temp
        self.quickMode = quick_mode

    def get_rooms(self):
        return self._rooms.values()

    def get_zones(self):
        return self._zones.values()

    def get_zone(self, zone_id: str):
        return self._zones[zone_id]

    def get_room(self, room_id: int):
        return self._rooms[room_id]

    def get_active_mode_zone(self, zone: Zone):
        """Holiday mode takes precedence over everything"""
        if self.holidayMode.active:
            return ActiveMode(self.holidayMode.targetTemperature, constant.HOLIDAY_MODE)

        """Global system quick mode takes over zone settings"""
        if self.quickMode and self.quickMode.boostMode.forZone:
            if self.quickMode.boostMode == QM_VENTILATION_BOOST:
                return ActiveMode(constant.FROST_PROTECTION_TEMP, self.quickMode.boostMode.name)

            if self.quickMode.boostMode == QM_ONE_DAY_AWAY:
                return ActiveMode(zone.targetMinTemperature, self.quickMode.boostMode.name)

            if self.quickMode.boostMode == QM_SYSTEM_OFF:
                return ActiveMode(constant.FROST_PROTECTION_TEMP, self.quickMode.boostMode.name)

            if self.quickMode.boostMode == QM_ONE_DAY_AT_HOME:
                return ActiveMode(zone.targetTemperature, self.quickMode.boostMode.name)

            if self.quickMode.boostMode == QM_PARTY:
                return ActiveMode(zone.targetTemperature, self.quickMode.boostMode.name)

            return None

        time_program = zone.get_current_time_program()
        if zone.quickVeto:
            return ActiveMode(time_program.temperature, time_program.mode)

        return ActiveMode(time_program.temperature, zone.operationMode, time_program.mode)

    def get_active_mode_room(self, room: Room):
        """Holiday mode takes precedence over everything"""
        if self.holidayMode.active:
            return ActiveMode(self.holidayMode.targetTemperature, constant.HOLIDAY_MODE)

        """Global system quick mode takes over zone settings"""
        if self.quickMode and self.quickMode.boostMode.forRoom:
            if self.quickMode.boostMode == QM_VENTILATION_BOOST:
                return ActiveMode(constant.FROST_PROTECTION_TEMP, self.quickMode.boostMode.name)

            if self.quickMode.boostMode == QM_ONE_DAY_AWAY:
                return ActiveMode(room.zone.targetMinTemperature, self.quickMode.boostMode.name)

            if self.quickMode.boostMode == QM_SYSTEM_OFF:
                return ActiveMode(constant.FROST_PROTECTION_TEMP, self.quickMode.boostMode.name)

            return None

        time_program = room.get_current_time_program()
        if room.quickVeto:
            return ActiveMode(time_program.temperature, time_program.mode)

        return ActiveMode(time_program.temperature, room.operationMode, time_program.mode)

    def get_active_mode_circulation(self, circulation: Circulation = None):
        if not circulation:
            circulation = self.circulation

        if self.holidayMode.active:
            return ActiveMode(0, constant.HOLIDAY_MODE)

        if self.quickMode and self.quickMode.boostMode.forCirculation:
            if self.quickMode.boostMode == QM_SYSTEM_OFF:
                return ActiveMode(0, self.quickMode.boostMode.name)

        time_program = circulation.get_current_time_program()
        return ActiveMode(0, circulation.operationMode, time_program.mode)

    def get_active_mode_hot_water(self, hot_water: HotWater = None):
        if not hot_water:
            hot_water = self.hotWater

        if self.holidayMode.active:
            return ActiveMode(self.holidayMode.targetTemperature, constant.HOLIDAY_MODE)

        if self.quickMode and self.quickMode.boostMode.forWaterHeater:
            if self.quickMode.boostMode == QM_HOTWATER_BOOST:
                return ActiveMode(hot_water.targetTemperature, self.quickMode.boostMode.name)

            if self.quickMode.boostMode == QM_SYSTEM_OFF:
                return ActiveMode(constant.FROST_PROTECTION_TEMP, self.quickMode.boostMode.name)

        time_program = hot_water.get_current_time_program()
        return ActiveMode(time_program.temperature, hot_water.operationMode, time_program.mode)



from .timeprogram import TimeProgram, TimeProgramDay, TimeProgramDaySetting
from .activemode import ActiveMode
from .boilerstatus import BoilerStatus
from .boostmode import BoostMode
from .quickmode import QuickMode
from .quickveto import QuickVeto
from .holidaymode import HolidayMode
from .component import Component
from .circulation import Circulation
from .hotwater import HotWater
from .device import Device
from .room import Room
from .zone import Zone
from .system import System
from .mapper import Mapper



from setuptools import setup, find_packages

setup(name='vr900-connector',
      version='0.0.25',
      description='Connector to handle vaillant vr900/vr920 data',
      long_description='Connector to handle vaillant vr900/vr920 data',
      url='https://github.com/thomasgermain/vr900-connector.git',
      author='Thomas Germain',
      author_email='thomas.germain@live.be',
      license='MIT',
      packages=find_packages(exclude=('tests', 'tests/*', '/tests', '/tests/*')),
      zip_safe=False,
      setup_requires=["pytest-runner"],
      install_requires=["requests", "responses", "pytest"])
